@@ -5,24 +5,15 @@
 
 from .util import Config, HdfsError
 from getpass import getuser
+from itertools import repeat
 from random import sample
 from requests_kerberos import HTTPKerberosAuth, OPTIONAL
+import os
 import os.path as osp
 import re
 import requests as rq
 
 
-def _hdfs_abspath(path, root):
-  """Return normalized absolute path.
-
-  :param path: HDFS path.
-  :param root: Root, used to allow relative paths.
-
-  """
-  if not (osp.isabs(path) or root and osp.isabs(root)):
-    raise HdfsError('Path %r is relative but no absolute root found.', path)
-  return osp.normpath(osp.join(root, path))
-
 def _on_error(response):
   """Callback when an API response has a non 2XX status code.
 
@@ -47,10 +38,6 @@ class _Request(object):
   :param verb: HTTP verb (`'GET'`, `'PUT'`, etc.).
   :param kwargs: Keyword arguments passed to the request handler.
 
-  Note that by default the `allow_redirects` keyword argument is set to `True`
-  and passed to the request handler. This is for convenience as all but 2 of
-  the API endpoints require it.
-
   """
 
   webhdfs_prefix = '/webhdfs/v1'
@@ -80,7 +67,7 @@ class _Request(object):
       url = '%s%s%s' % (
         client.url,
         self.webhdfs_prefix,
-        _hdfs_abspath(path, client.root),
+        client.resolve(path),
       )
       params['op'] = operation
       for key, value in client._params.items():
@@ -153,36 +140,6 @@ class Client(object):
     if proxy:
       self._params['doas'] = proxy
 
-  @classmethod
-  def _from_options(cls, class_name, options):
-    """Load client from options.
-
-    :param class_name: Client class name. Pass `None` for the base
-      :class:`Client` class.
-    :param options: Options dictionary.
-
-    """
-    class_name = class_name or 'Client'
-    try:
-      return cls.__registry__[class_name](**options)
-    except KeyError:
-      raise HdfsError('Unknown client class: %r', class_name)
-    except TypeError:
-      raise HdfsError('Invalid options: %r', options)
-
-  @classmethod
-  def from_alias(cls, alias, path=None):
-    """Load client associated with configuration alias.
-
-    :param alias: Alias name.
-    :param path: Path to configuration file. Defaults to `.hdfsrc` in the
-      current user's home directory.
-
-    """
-    path = path or osp.expanduser('~/.hdfsrc')
-    options = Config(path).get_alias(alias)
-    return cls._from_options(options.pop('client', None), options)
-
   # Raw API endpoints
 
   _append = _Request('PUT') # doesn't allow for streaming
@@ -205,6 +162,46 @@ class Client(object):
 
   # Exposed endpoints
 
+  def resolve(self, hdfs_path):
+    """Return absolute, normalized path, with special markers expanded.
+
+    :param hdfs_path: Remote path.
+
+    Currently supported markers:
+
+    * `'#LATEST'`: this marker gets expanded to the most recently updated file
+      or folder. They can be combined using the `'{N}'` suffix. For example,
+      `'foo/#LATEST{2}'` is equivalent to `'foo/#LATEST/#LATEST'`.
+
+    """
+    path = hdfs_path
+    if not (osp.isabs(path) or self.root and osp.isabs(self.root)):
+      raise HdfsError('Path %r is relative but no absolute root found.', path)
+    path = osp.normpath(osp.join(self.root, path))
+
+    def expand_latest(match):
+      """Substitute #LATEST marker."""
+      prefix = match.string[:match.start()]
+      suffix = ''
+      n = match.group(1) # n as in {N} syntax
+      for _ in repeat(None, int(n) if n else 1):
+        statuses = self._list_status(osp.join(prefix, suffix)).json()
+        candidates = sorted([
+          (-status['modificationTime'], status['pathSuffix'])
+          for status in statuses['FileStatuses']['FileStatus']
+        ])
+        if not candidates:
+          raise HdfsError('Cannot expand #LATEST. %r is empty.', prefix)
+        elif len(candidates) == 1 and candidates[0][1] == '':
+          raise HdfsError('Cannot expand #LATEST. %r is a file.', prefix)
+        suffix = osp.join(suffix, candidates[0][1])
+      return os.sep + suffix
+
+    # #LATEST expansion (could cache the pattern, but not worth it)
+    path = re.sub(r'/?#LATEST(?:{(\d+)})?(?=/|$)', expand_latest, path)
+
+    return path
+
   def content(self, hdfs_path):
     """Get content summary for a file or folder on HDFS.
 
@@ -349,14 +346,14 @@ class Client(object):
           for chunk in res.iter_content(chunk_size):
             yield chunk
         else:
-            buf = ''
-            for chunk in res.iter_content(chunk_size):
-              buf += chunk
-              splits = buf.split(buffer_char)
-              for part in splits[:-1]:
-                yield part
-              buf = splits[-1]
-            yield buf
+          buf = ''
+          for chunk in res.iter_content(chunk_size):
+            buf += chunk
+            splits = buf.split(buffer_char)
+            for part in splits[:-1]:
+              yield part
+            buf = splits[-1]
+          yield buf
       except GeneratorExit:
         pass
       finally:
@@ -402,7 +399,7 @@ class Client(object):
       a file, this method will raise :class:`~hdfs.util.HdfsError`.
 
     """
-    hdfs_dst_path = _hdfs_abspath(hdfs_dst_path, self.root)
+    hdfs_dst_path = self.resolve(hdfs_dst_path)
     res = self._rename(hdfs_src_path, destination=hdfs_dst_path)
     if not res.json()['boolean']:
       raise HdfsError('Remote path %r not found.', hdfs_src_path)
@@ -421,7 +418,7 @@ class Client(object):
     .. _FS: http://hadoop.apache.org/docs/r1.0.4/webhdfs.html#FileStatus
 
     """
-    hdfs_path = _hdfs_abspath(hdfs_path, self.root)
+    hdfs_path = self.resolve(hdfs_path)
     def _walk(dir_path, dir_status, depth):
       """Recursion helper."""
       yield dir_path, dir_status
@@ -441,6 +438,38 @@ class Client(object):
       for a in _walk(hdfs_path, status, depth):
         yield a
 
+  # Class loaders
+
+  @classmethod
+  def _from_options(cls, class_name, options):
+    """Load client from options.
+
+    :param class_name: Client class name. Pass `None` for the base
+      :class:`Client` class.
+    :param options: Options dictionary.
+
+    """
+    class_name = class_name or 'Client'
+    try:
+      return cls.__registry__[class_name](**options)
+    except KeyError:
+      raise HdfsError('Unknown client class: %r', class_name)
+    except TypeError:
+      raise HdfsError('Invalid options: %r', options)
+
+  @classmethod
+  def from_alias(cls, alias=None, path=None):
+    """Load client associated with configuration alias.
+
+    :param alias: Alias name.
+    :param path: Path to configuration file. Defaults to `.hdfsrc` in the
+      current user's home directory.
+
+    """
+    path = path or osp.expanduser('~/.hdfsrc')
+    options = Config(path).get_alias(alias)
+    return cls._from_options(options.pop('client', None), options)
+
 
 # Custom client classes
 # ---------------------
